# coding=utf-8
"""
阿里云相关API

Alibaba cloud API
"""
from . import pre_check, user_lang
from .. import requirePackage


class AliyunOSS:
    def __init__(
        self,
        ac_id=pre_check("aliyun_oss_acid"),
        ac_key=pre_check("aliyun_oss_ackey"),
        bucket_url=pre_check("aliyun_oss_df_endpoint"),
        df_bucket=pre_check("aliyun_oss_df_bucket"),
    ):
        """
        初始化并登陆阿里云对象存储

        Initializes and logs into ali cloud object storage

        :param ac_id: Access id
        :param ac_key: Access key
        :param bucket_url: oss-[地区 | location].aliyuncs.com
        :param df_bucket: 桶名称
        """
        self.ac_id = ac_id
        self.ac_key = ac_key
        self.bucket_url = bucket_url
        self.df_bucket = df_bucket
        self.progress, self.trans_id, self._progress_file_name = None, None, None
        self.auth = requirePackage("oss2", "Auth")(self.ac_id, self.ac_key)

    def get_func_table(self):
        """
        获取OSS类支持的操作

        Get the operations supported by the OSS class

        :return: 支持的函数字典
        """
        return {
            "-up": self.upload,
            "-rm": self.remove,
            "-dl": self.download,
            "-ls": self.list_bucket,
        }

    def _progress_bar(self, cur, total):
        """
        创建一个进度条并启动

        Create a progress bar and start

        :param cur: 当前完成量
        :param total: 总任务量
        :return: None
        """
        if not self.progress:
            from ..TuiTools.Bar import DataTransformBar

            self.progress = DataTransformBar(True if total else False)
            self.trans_id = self.progress.add_task(
                "Transform",
                total=total if total else -1,
                filename=self._progress_file_name,
            )
            self.progress.start()
            self.progress.start_task(self.trans_id)
        self.progress.update(self.trans_id, completed=cur)

    def upload(self, filePath: str, bucket: str = None, key: str = None):
        """
        上传文件（支持断点续传）

        Upload file (support breakpoint continuation)

        :param filePath: 文件路径
        :param bucket: 桶名称，缺省使用self.df_bucket
        :return: None
        """
        import os
        from .. import qs_default_console, qs_info_string, dir_char
        from ..SystemTools import get_core_num

        bucket = bucket if bucket else self.df_bucket
        bucket = requirePackage("oss2", "Bucket")(self.auth, self.bucket_url, bucket)
        filePath = filePath.strip()
        self._progress_file_name = os.path.basename(filePath)
        requirePackage("oss2", "resumable_upload")(
            bucket,
            key if key else filePath.replace(dir_char, "/"),
            filePath,
            num_threads=get_core_num() * 4,
            progress_callback=self._progress_bar,
        )
        self.progress.stop()
        qs_default_console.print(
            qs_info_string, "Transform Completed!" if user_lang != "zh" else "传输完成!"
        )

    def download(self, filename: str, bucket: str = None):
        """
        下载文件（支持断点续传）

        Download file (support breakpoint continuation)

        :param filename: 文件在bucket中的路径
        :param bucket: 桶名称，缺省使用self.df_bucket
        :return: None
        """
        import os
        from .. import qs_default_console, qs_info_string
        from ..SystemTools import get_core_num

        bucket = bucket if bucket else self.df_bucket
        bucket = requirePackage("oss2", "Bucket")(self.auth, self.bucket_url, bucket)
        self._progress_file_name = os.path.basename(filename)
        requirePackage("oss2", "resumable_download")(
            bucket,
            filename,
            filename,
            num_threads=get_core_num() * 4,
            progress_callback=self._progress_bar,
        )
        self.progress.stop()
        qs_default_console.print(
            qs_info_string, "Transform Completed!" if user_lang != "zh" else "传输完成!"
        )

    def remove(self, filePath: str, bucket: str = None):
        """
        删除文件

        delete file

        :param filePath: 文件在bucket中的路径
        :param bucket: 桶名称，缺省使用self.df_bucket
        :return: None
        """
        bucket = bucket if bucket else self.df_bucket
        bucket = requirePackage("oss2", "Bucket")(self.auth, self.bucket_url, bucket)
        bucket.delete_object(filePath)

    def list_bucket(self, bucket: str = None):
        """
        打印桶中的文件表

        Print the file table in the bucket

        :param bucket: 桶名称，缺省使用self.df_bucket
        :return: None
        """
        from .. import qs_default_console, qs_info_string
        from ..NetTools.NormalDL import size_format
        from ..TuiTools.Table import qs_default_table
        from rich.text import Text

        bucket = bucket if bucket else self.df_bucket
        ls = requirePackage("oss2", "Bucket")(self.auth, self.bucket_url, bucket)
        tb = qs_default_table(
            ["File", "Size"] if user_lang != "zh" else ["文件", "体积"], title="Aliyun OSS"
        )
        prefix = dict()
        for obj in requirePackage("oss2", "ObjectIterator")(ls):
            if "/" in obj.key:
                prefix[obj.key[obj.key[: obj.key.index("/")]]] = 0
            tb.add_row(
                Text(obj.key, justify="left"),
                Text(size_format(obj.size, True), justify="right"),
            )
        qs_default_console.print(
            qs_info_string,
            "Bucket Url:" if user_lang != "zh" else "桶链接:",
            "https://" + bucket + "." + self.bucket_url + "/",
        )
        qs_default_console.print(tb, justify="center")



# coding=utf-8
"""
调用各种图像处理工具

Call various image processing tools
"""
import sys
import os
from . import qs_default_console, qs_error_string


def set_img_background():
    """替换图片颜色 | Replace color in image"""
    from .ImageTools.ColorTools import transport_background, get_color_from_str

    try:
        img = sys.argv[2]
        if img == "-help":
            raise IndexError
        if not os.path.exists(img):
            qs_default_console.log(qs_error_string, "No Such File: %s" % img)
            return
        to = sys.argv[3]
        try:
            frm = sys.argv[4]
        except IndexError:
            frm = "0,0,0,0"
    except IndexError:
        qs_default_console.log(
            qs_error_string,
            "Usage: qs stbg <picture> <to_color> [from_color: default transparency]",
        )
        return
    else:
        to = get_color_from_str(to)
        frm = get_color_from_str(frm)
        img = transport_background(img, to, frm)
        iname = sys.argv[2].split(".")
        iname = iname[0] + "_stbg." + "".join(iname[1:])
        img.save(iname)


def fmt_img_color():
    from .ImageTools.ColorTools import formatOneColor, get_color_from_str

    try:
        img = sys.argv[2]
        if img == "-help":
            raise IndexError
        if not os.path.exists(img):
            qs_default_console.log(qs_error_string, "No Such File: %s" % img)
            return
        to = sys.argv[3]
        try:
            exp = sys.argv[4:]
        except IndexError:
            exp = "0,0,0,0"
    except IndexError:
        qs_default_console.log(
            qs_error_string,
            "Usage: qs fmti <picture> <to_color> [except_color: default transparency]",
        )
        return
    else:
        to = get_color_from_str(to)
        exp = [get_color_from_str(i) for i in exp]
        img = formatOneColor(img, to, exp)
        iname = sys.argv[2].split(".")
        iname = iname[0] + "_fmt." + "".join(iname[1:])
        img.save(iname)


def v2gif():
    """视频转gif | video to gif"""
    from .ImageTools.VideoTools import video_2_gif

    try:
        video = sys.argv[2]
        sz, fps = None, None
        if len(sys.argv) > 3:
            for i in sys.argv[3:]:
                try:
                    if "," in i:
                        sz = tuple([int(j) for j in i.split(",")])
                    else:
                        fps = int(i)
                except:
                    raise IndexError
        sz = (
            tuple([int(i) for i in sys.argv[3].split(",")])
            if len(sys.argv) > 3 and "," in sys.argv[3]
            else None
        )
    except IndexError:
        qs_default_console.log(
            qs_error_string, "Usage: qs v2gif <*.mp4> [width,height] [fps]"
        )
        return
    else:
        (
            video_2_gif(video, sz, fps)
            if sz and fps
            else (
                video_2_gif(video, sz)
                if sz
                else video_2_gif(video, fps=fps) if fps else video_2_gif(video)
            )
        )


def remove_audio():
    """删除视频的音频 | remove audio in mp4"""
    from .ImageTools.VideoTools import rm_audio

    try:
        videos = sys.argv[2:]
    except IndexError:
        qs_default_console.log(qs_error_string, "Usage: qs rmaudio <video...>")
    else:
        for video in videos:
            rm_audio(video)


def v2mp4():
    """视频转mp4 | transfer video to mp4"""
    from .ImageTools.VideoTools import tomp4

    try:
        videos = sys.argv[2:]
    except IndexError:
        qs_default_console.log(qs_error_string, "Usage: qs v2mp4 <video...>")
    else:
        for video in videos:
            tomp4(video)


def v2mp3():
    """提取视频音频为mp3 | Extract the audio from the video and save it in MP3 format"""
    from .ImageTools.VideoTools import video_2_mp3

    try:
        videos = sys.argv[2:]
    except IndexError:
        qs_default_console.log(qs_error_string, "Usage: qs v2mp3 <video...>")
    else:
        for video in videos:
            video_2_mp3(video)


def icat():
    """Mac::iTerm下预览图片 | Preview the picture under Mac::iTerm"""
    try:
        path = sys.argv[2]
        is_url = "-u" in sys.argv or (
            not os.path.exists(path)
            and (path.startswith("http://") or path.startswith("https://"))
        )
        if not os.path.exists(path) and not is_url:
            qs_default_console.log(qs_error_string, "No such file:", path)
            raise FileNotFoundError
    except:
        qs_default_console.log(
            qs_error_string, "Usage: qs icat <img path/url> [-u if is url]"
        )
    else:
        from .ImageTools.ImagePreview import image_preview

        image_preview(path) if not is_url else image_preview(path, is_url)


def i2png():
    try:
        imgs = sys.argv[2:]
    except IndexError:
        qs_default_console.log(qs_error_string, "Usage: qs i2png <imgs...>")
    else:
        from .ImageTools.ImageTools import topng

        for imgPath in imgs:
            topng(imgPath)


def i2jpg():
    try:
        imgs = sys.argv[2:]
    except IndexError:
        qs_default_console.log(qs_error_string, "Usage: qs i2jpg <imgs...>")
    else:
        from .ImageTools.ImageTools import tojpg

        for imgPath in imgs:
            tojpg(imgPath)


def i2pdf():
    try:
        imgs = sys.argv[2:]
    except IndexError:
        qs_default_console.log(qs_error_string, "Usage: qs i2pdf <imgs...>")
    else:
        from .ImageTools.ImageTools import topdf

        for imgPath in imgs:
            topdf(imgPath)


def vsta():
    from .ImageTools.VideoTools import set_video_audio

    try:
        set_video_audio(*sys.argv[2:])
    except Exception as e:
        qs_default_console.log(
            qs_error_string, f"{repr(e)}\nUsage: qs vsta <video> <audio>"
        )



from ... import qs_config, requirePackage

_chatbot = None

def _msg_to_iterator(msg):
    yield msg

def _help(_):
    return _msg_to_iterator(
"""Usage: Type your message to chat with the bot

|Command|Description|
|---|---|
|/help|Show this message|
|/save <filename>|Save the conversation to a file|
""")

def save_conversation(filename: str):
    global _chatbot

    if _chatbot:
        _chatbot.save_conversation(filename)
    return _msg_to_iterator("Conversation saved to file: " + filename)


cmd = {
    '/help': _help,
    '/save': save_conversation
}

def ChatGPT(prompt: str, system_prompt: str = None, record_history=True, model=None):
    global _chatbot

    if not _chatbot:
        _config = qs_config.basicSelect("gpt")
        api_supporter = _config["index"]
        if not system_prompt:
            system_prompt = qs_config.basicSelect("gpt")["prompt"].items()[0][1]
        if not model:
            model = _config["model"]
        _chatbot = requirePackage(".API.GPT." + api_supporter, "create_bot")(
            model,
            system_prompt=system_prompt,
            record_history=record_history,
        )

    if prompt.startswith("/"):
        # parse command
        cmd_name, *args = prompt.split()
        if cmd_name in cmd:
            return cmd[cmd_name](' '.join(args))
        else:
            return _msg_to_iterator("Unknown command: " + cmd_name)
        
    return _chatbot.ask_stream(prompt)



"""
本模块提供了一些使用numba包的工具，会引导用户安装numba包，如果用户没有安装numba包，会自动跳过使用numba包的功能

This module provides some tools using the numba package, which will guide the user to install the numba package. If the user does not install the numba package, the function using the numba package will be skipped automatically
"""

from .. import requirePackage
jit = requirePackage("numba", "jit")
function_cache = {}

def cut_string(string: str, length: int, ignore_charset: list = []) -> list:
    """
    每隔l个字符切分字符串
    引用numba包会有额外的代价, 但是速度会快很多

    :param string: 字符串
    :param length: 切分长度
    :param ignore_charset: 忽略的字符集
    :return: 切分后产生的list
    """
    if 'cut_string' not in function_cache:
        function_cache['cut_string'] = jit(requirePackage(".", "cut_string"), nopython=True) if jit else requirePackage(".", "cut_string")
    return function_cache['cut_string'](string, length, ignore_charset)



from . import pre_check
from .. import requirePackage

pushdeer_api_info = pre_check('pushdeer')

server = pushdeer_api_info["url"]
key = pushdeer_api_info['key']
pushdeer = requirePackage('pypushdeer', 'PushDeer')(server=server, pushkey=key)


def send_text(text):
    """
    推送文本消息
    :param text: 文本内容
    :return:
    """
    return pushdeer.send_text(text)


def send_image(image_path):
    """
    推送图片消息
    :param image_path: 图片路径
    :return:
    """
    return pushdeer.send_image(image_path)

def send_markdown(content):
    """
    推送Markdown消息
    :param title: 标题
    :param content: 内容
    :return:
    """
    return pushdeer.send_markdown(content)



from . import pre_check

serper_api_key = pre_check('serper')


def __do_request(query, action):
    import json
    import requests

    url = f"https://google.serper.dev/{action}"
    payload = json.dumps({
        "q": query
    })
    headers = {
        'X-API-KEY': serper_api_key,
        'Content-Type': 'application/json'
    }
    response = requests.request("POST", url, headers=headers, data=payload)
    return response.json()


def search(query):
    return __do_request(query, "search")

def images(query):
    return __do_request(query, "images")

def videos(query):
    return __do_request(query, "videos")

def places(query):
    return __do_request(query, "places")

def maps(query):
    return __do_request(query, "maps")

def news(query):
    return __do_request(query, "news")

def shopping(query):
    return __do_request(query, "shopping")

def scholar(query):
    return __do_request(query, "scholar")

def patents(query):
    return __do_request(query, "patents")

def autocomplete(query):
    return __do_request(query, "autocomplete")



import os
import time
import pickle
import atexit

class QsCache:
    """
    qs存储在~/.qs_cache的缓存
    """
    def __init__(self, cachePath: str):
        self.path = cachePath
        self.cache_table_path = os.path.join(self.path, 'qs_cache_table')
        if not os.path.exists(cachePath):
            os.mkdir(cachePath)
        if not os.path.exists(self.cache_table_path):
            self.cache_table = {}
        else:
            with open(self.cache_table_path, 'rb') as f:
                self.cache_table = pickle.load(f)
        need_remove = []
        for item in self.cache_table:
            if not os.path.exists(self.cache_table[item]['path']):
                need_remove.append(item)
        for item in need_remove:
            del self.cache_table[item]
        atexit.register(self.exit)

    def get(self, key: str):
        if key not in self.cache_table:
            return None
        self.cache_table[key]['used'] = time.time()
        with open(self.cache_table[key]['path'], "rb") as f:
            return pickle.load(f)

    def set(self, key: str, value, expire_days: int = 0):
        item_path = os.path.join(self.path, key)
        with open(item_path, "wb") as f:
            pickle.dump(value, f)
        
        self.cache_table[key] = {
            'used': time.time(),
            'expire': expire_days * 24 * 60 * 60,
            'path': item_path
        }

    def delete(self, key: str):
        if key in self.cache_table:
            item_path = self.cache_table[key]['path']
            if os.path.exists(item_path):
                os.remove(item_path)
            del self.cache_table[key]
    
    def save_table(self):
        with open(self.cache_table_path, 'wb') as f:
            pickle.dump(self.cache_table, f)

    def exit(self):
        # auto delete item expired
        remove_key = []
        for key in self.cache_table:
            if self.cache_table[key]['expire'] != 0:
                if time.time() - self.cache_table[key]['used'] > self.cache_table[key]['expire']:
                    remove_key.append(key)
        for key in remove_key:
            self.delete(key)
        self.save_table()



# coding=utf-8
"""
命令入口

Command entry
"""
import sys
from .funcList import *
from . import requirePackage


def qs_help(rep: str = ""):
    """输出菜单 | Output menu"""
    if not rep or rep not in ["basic", "system", "net", "api", "image"]:
        menu_show(
            [
                ["basic", "u, a, f, cal, time, pcat.."],
                ["system", "top, \[mk/un]\[zip/tar/7z].."],
                ["net", "http, dl, netinfo, wifi..."],
                ["api", "trans, smms, rmbg, loli..."],
                ["image", "stbg, v2gif, v2mp3, v2mp4."],
            ]
        )
        qs_default_console.print(
            f"\n[bold]{lang_detector['tutorial']}[/]\n", justify="center"
        )
        qs_default_console.print("[bold magenta]qs lesson[/]", justify="center")
    else:
        menu_table(rep)


def main():
    """执行命令 | Execute"""
    debug_flag = "--qs-debug" in sys.argv
    if debug_flag:
        sys.argv.remove("--qs-debug")
    help_flag = "--help" in sys.argv
    if help_flag:
        sys.argv.remove("--help")
    if len(sys.argv) >= 2:
        try:
            func_name = sys.argv[1]
            if func_name not in cmd_table:
                qs_help(func_name)
            else:
                if help_flag:
                    return command_help(func_name)
                func_table = cmd_table[func_name]
                requirePackage(func_table["self"], func_table[func_name][0])()
        except Exception as e:
            from . import qs_default_console

            if debug_flag:
                qs_default_console.print_exception()
                import traceback
                from . import live_show
                from .API.GPT import ChatGPT

                live_show(
                    ChatGPT(
                        traceback.format_exc(),
                        system_prompt="You are a professional computer engineer, please help me analyze the error, and tell me how to solve it.",
                    )
                )
            else:
                from . import qs_error_string

                qs_default_console.log(qs_error_string, repr(e))
    else:
        qs_help()


if __name__ == "__main__":
    main()



from .GPT import ChatGPT
from .. import user_lang, qs_config

translate_model = qs_config.basicSelect("gpt").get("translate", None)
if not translate_model:
    from .. import qs_default_console
    qs_default_console.print("You can set \".qsrc['basic_settings']['gpt']['translate']\" to specify the translation model.")


def translate(text, target_lang=user_lang.lower()):
    prompt = f"Please translate into {target_lang} (avoid explaining the original text): {text}"
    response = ChatGPT(prompt, system_prompt="You are a professional, authentic translation engine. You only return the translated text, without any explanations.", record_history=False, model=translate_model)
    return response



from ... import user_lang, qs_default_console, qs_error_string, qs_config
from .common import GPTBotBase
import time

API_KEY = qs_config.basicSelect("gpt")["support"].get("alapi", None)
if not API_KEY:
    raise RuntimeError("No API key for POE Chatbot.")

lang_table = {
    'zh': 'Chinese',
    'en': 'English',
    'fra': 'French',
    'ru': 'Russian',
    'spa': 'Spanish',
    'ara': 'Arabic',
}

import requests
import json

class AlapiChatbot(GPTBotBase):
    def __init__(
            self, 
            system_prompt=f"You are ChatGPT, a large language model trained by OpenAI. Respond conversationally with Markdown format and using {lang_table[user_lang]} Language.",
            record_history=True
        ):
        super().__init__(system_prompt=system_prompt, record_history=record_history)

        from ..alapi import v2_url

        self.api_key = API_KEY
        self.url = v2_url
    
    def save_conversation(self, file_path: str):
        with open(file_path, "w", encoding="utf-8") as f:
            for message in self.messages:
                f.write(f"{message['role']}\n\n{message['content']}\n---\n")

    def ask(self, prompt):
        self.messages.append(
            {
                "role": "user",
                "content": prompt,
            }
        )
        while res := (
            requests.post(
                self.url + "chatgpt/pro",
                json={
                    "token": self.api_key,
                    "message": self.messages,
                },
            )
            .json()
        ):
            if res['code'] == 422 or not res.get('data'):
                qs_default_console.print(qs_error_string, res)
                time.sleep(3)
                continue
            else:
                break
        res = res.get("data").get("content")
        if self.record_history:
            self.messages.append(
                {
                    "role": "assistant",
                    "content": res,
                }
            )
        return res

    def ask_stream(self, prompt):  # not support now
        self.messages.append(
            {
                "role": "user",
                "content": prompt,
            }
        )
        post_stream = requests.post(
            self.url + "chatgpt/stream",
            json={
                "token": self.api_key,
                "message": self.messages,
            },
            stream=True,
        )
        total_res = ""
        for line in post_stream.iter_lines(decode_unicode=True):
            line = line.strip()
            if not line:
                continue
            res = json.loads(line)
            if res['success']:
                total_res = res['text']
                yield total_res
        if self.record_history:
            self.messages.append(
                {
                    "role": "bot",
                    "content": total_res,
                }
            )

def create_bot(
    model: str,
    system_prompt: str = f"You are ChatGPT, a large language model trained by OpenAI. Respond conversationally with Markdown format and using {lang_table[user_lang]} Language.",
    record_history=True
):
    return AlapiChatbot(model, system_prompt, record_history)



# coding=utf-8
"""
压缩与解压缩各种文件

Compress and decompress all kinds of files
"""
import io
import os
import sys

from .. import (
    user_lang,
    qs_default_console,
    qs_warning_string,
    requirePackage,
)


def get_compress_package_name():
    """
    从命令行参数中，解析并返回压缩包名称

    From the command line arguments, the package name is parsed and returned.

    :return: 压缩包名称，合法的文件列表 | Compressed package name, a list of legitimate files.
    """
    file_names = sys.argv[2:]
    if not file_names:
        exit("No enough parameters")
    if len(file_names) > 1:
        name = "pigeonhole"
    else:
        name = os.path.basename(file_names[0]).split(".")[0]
    ls = []
    for file_name in file_names:
        if os.path.exists(file_name):
            ls.append(file_name)
        else:
            qs_default_console.print(
                qs_warning_string,
                f"{file_name} not found" if user_lang != "zh" else f"{file_name} 未找到",
            )
    return name, ls


def checkIsProtocolFile(protocol, verify_func, path):
    """
    验证压缩包文件

    Verify compressed package file

    :param protocol: 压缩协议 | compress protocol
    :param path: 文件路径 | File Path
    :return:
    """

    protocol_name = {
        "tarfile": "tar",
        "zipfile": "zip",
        "rarfile": "rar",
        "py7zr": "7z",
    }

    if os.path.exists(path):
        if not requirePackage(protocol, verify_func)(path):
            raise TypeError(
                f"{path} "
                + (
                    f"Not recognized by {protocol_name[protocol]} protocol"
                    if user_lang != "zh"
                    else f"无法被{protocol_name[protocol]}协议识别"
                )
            )
    else:
        raise FileNotFoundError


class _NormalCompressedPackage:
    """
    通用压缩协议类，如果你不懂它是做什么的，请不要调用它

    General compression protocol class, if you do not understand what it does, please do not call it
    """

    def __init__(self, protocol, verify_func, obj_name, path: str, mode="r"):
        """
        通用压缩协议类初始化

        General compression protocol class initialization

        :param protocol: 压缩协议包 | General compression protocol packages
        :param verify_func: 验证函数 | Verification function
        :param obj_name: 对象名称 | Object name
        :param path: 压缩包路径 | compression package path
        :param mode: 读写模式 | 'r' or 'w', which 'r' means read and 'w' means write
        """
        self._protocol = protocol
        self.path = path
        if mode not in ("r", "w"):
            raise ValueError("Requires mode 'r', 'w'")
        if mode == "r":
            checkIsProtocolFile(protocol, verify_func, path)
            if protocol != "rarfile":
                self.src = requirePackage(protocol, obj_name)(path, "r")
            else:
                self.src = requirePackage(protocol, obj_name)(path)
            self.mode = True
        elif mode == "w":
            if protocol != "rarfile":
                self.src = requirePackage(protocol, obj_name)(path, "w")
            else:
                raise NotImplementedError(
                    "qs not support to create rar file because `RarFile`"
                )
            self.mode = False

    def add_file(self, path):
        """
        向压缩包添加文件（工作在'w'模式下）

        Add file to the compressed package (works in 'w' mode)

        :param path: 文件路径 | File path
        :return: None
        """
        if self.mode:
            raise io.UnsupportedOperation
        if not os.path.exists(path):
            raise FileNotFoundError
        if os.path.isdir(path):
            for root, dirs, files in os.walk(path):
                for file in files:
                    if self._protocol == "tarfile":
                        self.src.add(os.path.join(root, file))
                    elif self._protocol in ["zipfile", "py7zr"]:
                        self.src.write(os.path.join(root, file))
        else:
            if self._protocol == "tarfile":
                self.src.add(path)
            elif self._protocol in ["zipfile", "py7zr"]:
                self.src.write(path)

    def extract(self):
        """
        解压缩 | extract

        :return: None
        """
        if self.mode:
            if self._protocol in ["tarfile", "rarfile", "py7zr"]:
                self.src.extractall()
            elif self._protocol == "zipfile":
                from pathlib import Path

                for fn in self.src.namelist():
                    path = Path(self.src.extract(fn))
                    try:
                        path.rename(fn.encode("cp437").decode("utf-8"))
                    except:
                        try:
                            path.rename(fn.encode("cp437").decode("gbk"))
                        except Exception as e:
                            qs_default_console.log(qs_warning_string, repr(e))

            else:
                raise NotImplementedError
            self.save()
        else:
            raise io.UnsupportedOperation

    def save(self):
        """
        保存 | Save

        :return: None
        """
        self.src.close()


class Tar(_NormalCompressedPackage):
    def __init__(self, path, mode="r"):
        """
        Tar协议初始化

        Tar protocol initialization.

        :param path: 压缩包路径 | The package path is compressed.
        :param mode: 工作模式 | Working mode ('read' or 'write')
        """
        super().__init__("tarfile", "is_tarfile", "TarFile", path, mode)

    def add_file(self, path):
        """
        向压缩包添加文件（工作在'write'模式下）

        Add files to the compressed package (works in 'write' mode)

        :param path: 文件路径
        :return: None
        """
        return super().add_file(path)

    def extract(self):
        """
        解压缩

        extract

        :return: None
        """
        return super().extract()

    def save(self):
        """保存 | save"""
        return super().save()


class Zip(_NormalCompressedPackage):
    def __init__(self, path, mode="r"):
        """
        Zip协议初始化

        Zip protocol initialization.

        :param path: 压缩包路径 | The package path is compressed.
        :param mode: 工作模式 | Working mode ('read' or 'write')
        """
        super().__init__("zipfile", "is_zipfile", "ZipFile", path, mode)

    def add_file(self, path):
        """
        向压缩包添加文件（工作在'w'模式下）

        Add files to the compressed package (works in 'write' mode)

        :param path: 文件路径
        :return: None
        """
        return super().add_file(path)

    def extract(self):
        """
        解压缩

        extract

        :return: None
        """
        return super().extract()

    def save(self):
        """保存 | save"""
        return super().save()


class Rar(_NormalCompressedPackage):
    def __init__(self, path, mode="r"):
        """
        Rar协议初始化

        Rar protocol initialization.

        :param path: 压缩包路径 | The package path is compressed.
        :param mode: 工作模式 | Working mode ('read' or 'write')
        """
        super().__init__("rarfile", "is_rarfile", "RarFile", path, mode)

    def add_file(self, path):
        """
        向压缩包添加文件（工作在'w'模式下）

        Add files to the compressed package (works in 'write' mode)

        :param path: 文件路径
        :return: None
        """
        return super().add_file(path)

    def extract(self):
        """
        解压缩

        extract

        :return: None
        """
        return super().extract()

    def save(self):
        """保存 | save"""
        return super().save()


class SevenZip(_NormalCompressedPackage):
    def __init__(self, path, mode="r"):
        """
        7z协议初始化

        7z protocol initialization.

        :param path: 压缩包路径 | The package path is compressed.
        :param mode: 工作模式 | Working mode ('read' or 'write')
        """
        super().__init__("py7zr", "is_7zfile", "SevenZipFile", path, mode)

    def add_file(self, path):
        """
        向压缩包添加文件（工作在'w'模式下）

        Add files to the compressed package (works in 'write' mode)

        :param path: 文件路径
        :return: None
        """
        return super().add_file(path)

    def extract(self):
        """
        解压缩

        extract

        :return: None
        """
        return super().extract()

    def save(self):
        """保存 | save"""
        return super().save()



from ... import requirePackage, user_lang, qs_config
from .common import GPTBotBase

API_KEY = qs_config.basicSelect("gpt")["support"].get("poe", None)
if not API_KEY:
    raise RuntimeError("No API key for POE Chatbot.")

import asyncio
import queue
import concurrent

get_bot_response = requirePackage(
    "fastapi_poe.client", "get_bot_response", real_name="fastapi-poe"
)
ProtocolMessage = requirePackage(
    "fastapi_poe.types", "ProtocolMessage", real_name="fastapi-poe"
)


lang_table = {
    "zh": "Chinese",
    "en": "English",
    "fra": "French",
    "ru": "Russian",
    "spa": "Spanish",
    "ara": "Arabic",
}


class AsyncToSyncIterator:
    def __init__(self, async_gen):
        self.queue = queue.Queue(maxsize=1)
        self.executor = concurrent.futures.ThreadPoolExecutor(max_workers=1)

        def run():
            async def _run():
                try:
                    async for item in async_gen:
                        self.queue.put(item)
                finally:
                    self.queue.put(StopIteration)

            loop = asyncio.new_event_loop()
            asyncio.set_event_loop(loop)
            loop.run_until_complete(_run())
            loop.close()

        self.executor.submit(run)

    def __iter__(self):
        return self

    def __next__(self):
        item = self.queue.get()
        if item is StopIteration:
            raise StopIteration
        return item


class POEChatbot:
    def __init__(self, model, system_prompt, record_history=True):
        self.model = model
        self.messages = [ProtocolMessage(role="system", content=system_prompt)]
        self.record_history = record_history

    async def _ask_stream(self, prompt):
        self.messages.append(ProtocolMessage(role="user", content=prompt))
        if len(self.messages) > 4096:
            self.messages.pop(0)
        total_res = ""
        async for res in get_bot_response(
            self.messages, bot_name=self.model, api_key=API_KEY
        ):
            total_res += res.text
            yield total_res
        if self.record_history:
            self.messages.append(ProtocolMessage(role="bot", content=total_res))

    def ask_stream(self, prompt):
        return AsyncToSyncIterator(self._ask_stream(prompt))


def create_bot(
    model: str,
    system_prompt: str = f"You are ChatGPT, a large language model trained by OpenAI. Respond conversationally with Markdown format and using {lang_table[user_lang]} Language.",
    record_history=True
):
    return POEChatbot(model, system_prompt, record_history)



# coding=utf-8
"""
调用各种网络工具

Call various network tools
"""
import sys
from . import qs_default_console, requirePackage, lang_detector

def upgrade():
    """
    更新qs

    Upgrade qs
    """
    from . import (
        qs_default_console,
        qs_info_string,
        qs_default_status,
    )
    with qs_default_status(lang_detector['updating']):
        requirePackage("QuickStart_Rhy", keep_latest=True)
    qs_default_console.print(qs_info_string, lang_detector['updated'])


def m3u8_dl(url):
    """
    下载m3u8

    Download *.m3u8
    """
    from .NetTools.M3u8DL import M3U8DL

    M3U8DL(url, url.split(".")[-2].split("/")[-1]).download()


def download():
    """
    qs下载引擎，使用--video or -v使用youtube-dl下载视频

    Qs download engine, use --video or -v use the default video download engine download
    """
    if any([i in sys.argv for i in ["-h", "-help", "--help"]]):
        from . import user_lang, qs_default_console, qs_info_string

        qs_default_console.print(
            qs_info_string,
            'Usage: "qs dl [url...]"\n'
            "  [--video] | [-v]  :-> download video (use youtube-dl)\n"
            "  [--proxy] | [-px] :-> use default proxy set in ~/.qsrc\n"
            "  [--name fileName] :-> Set File Name\n"
            "  [--referer] | [-e] :-> Set Referer\n"
            if user_lang != "zh"
            else '使用: "qs dl [链接...]"\n'
            "  [--video] | [-v]  :-> 使用youtube-dl下载视频\n"
            "  [--proxy] | [-px] :-> 使用配置表中的默认代理下载\n"
            "  [--name fileName] :-> 设置文件名\n"
            "  [--referer] | [-e] :-> 设置Referer\n",
        )
        return
    global _real_main
    ytb_flag = "--video" in sys.argv or "-v" in sys.argv
    use_proxy = "--proxy" in sys.argv or "-px" in sys.argv
    other_args = []
    set_name = None
    set_referer = None
    if "--name" in sys.argv:
        set_name = sys.argv[sys.argv.index("--name") + 1]
    if "--referer" in sys.argv or "-e" in sys.argv:
        set_referer = (
            sys.argv[sys.argv.index("--referer") + 1]
            if "--referer" in sys.argv
            else sys.argv[sys.argv.index("-e") + 1]
        )
        if not set_referer.endswith("/"):
            set_referer += "/"
    if ytb_flag or use_proxy or set_name or set_referer:
        [
            sys.argv.remove(i) if i in sys.argv else None
            for i in [
                "--video",
                "-v",
                "--proxy",
                "-px",
                "--name",
                "--referer",
                "-e",
                set_name,
                set_referer,
            ]
        ]
    nxt_flag = False
    for item in sys.argv[2:]:
        if item.startswith("-") or nxt_flag:
            if nxt_flag:
                nxt_flag = False
            if item.startswith("--"):
                nxt_flag = True
            other_args.append(item)

    [sys.argv.remove(item) for item in other_args]
    urls = sys.argv[2:]
    if not urls:
        from . import requirePackage

        pyperclip = requirePackage("pyperclip")
        urls = pyperclip.paste().split()
    if urls:
        if ytb_flag:
            from . import requirePackage
            from youtube_dl import _real_main

            _real_main = requirePackage("youtube_dl", "_real_main")
        from .NetTools.NormalDL import normal_dl
        from . import qs_config

        for url in urls:
            if url.endswith(".m3u8"):
                m3u8_dl(url)
            else:
                if use_proxy:
                    normal_dl(
                        url,
                        set_name=set_name,
                        set_proxy=qs_config.basicSelect("default_proxy"),
                        set_referer=set_referer,
                    ) if not ytb_flag else _real_main(
                        [
                            url,
                            "--proxy",
                            qs_config.basicSelect("default_proxy"),
                            "--merge-output-format",
                            "mp4",
                        ]
                        + other_args
                    )
                else:
                    normal_dl(
                        url, set_name=set_name, set_referer=set_referer
                    ) if not ytb_flag else _real_main(
                        [url, "--merge-output-format", "mp4"] + other_args
                    )
    else:
        from . import user_lang, qs_default_console, qs_error_string

        qs_default_console.log(
            qs_error_string, "No url found!" if user_lang != "zh" else "无链接输入"
        )

def multi_download():
    urls = sys.argv[2:]

    from .NetTools.MultiSingleDL import multi_single_dl
    multi_single_dl(urls)

def http():
    """
    开启http服务

    Turn on the http service.
    """
    url = ""
    if len(sys.argv) > 2:
        ip, port = sys.argv[2].split(":")
        port = int(port)
        if "-bind" in sys.argv:
            try:
                url = sys.argv[sys.argv.index("-bind") + 1]
                from .NetTools import formatUrl

                url = formatUrl(url)
            except IndexError:
                print(f"Usage: qs http ip:port -bind url")
                exit(0)
    else:
        from .NetTools import get_ip

        ip = get_ip()
        port = 8000
    if not ip:
        exit("get ip failed!")
    from .NetTools.HttpServer import HttpServers

    HttpServers(ip, port, url).start()


def netinfo():
    """
    通过域名或ip查询ip信息

    Query ip information via domain name or ip.
    """
    import socket
    import urllib.parse
    from . import user_lang, qs_default_console, qs_error_string, requirePackage

    from .API.alapi import ip_info

    def print_ip_info(info_ls):
        from .TuiTools.Table import qs_default_table

        table = qs_default_table(
            [
                {"header": "ip", "justify": "center", "style": "bold magenta"},
                {"header": "运营商", "justify": "center"},
                {"header": "地址", "justify": "center"},
                {"header": "经纬", "justify": "center"},
            ]
            if user_lang == "zh"
            else [
                {"header": "ip", "justify": "center", "style": "bold magenta"},
                {"header": "isp", "justify": "center"},
                {"header": "pos", "justify": "center"},
                {"header": "location", "justify": "center"},
            ],
            "NetInfo Results\n",
        )
        for info in info_ls:
            table.add_row(
                info["ip"],
                info["isp"].strip() if "isp" in info else "未知",
                info["pos"],
                str(info["location"])[1:-1].replace("'", ""),
            )
        qs_default_console.print(table, justify="center")

    urls = sys.argv[2:] if len(sys.argv) > 2 else []
    if not urls:
        try:
            urls += (
                requirePackage("pyperclip").paste().strip().split() if not urls else []
            )
        except:
            from rich.prompt import Prompt

            urls = (
                Prompt.ask(
                    "Sorry, but your system is not supported by `pyperclip`\nSo you need input content manually: "
                    if user_lang != "zh"
                    else "抱歉，但是“pyperclip”不支持你的系统\n，所以你需要手动输入内容:"
                )
                .strip()
                .split()
            )
    if not urls:
        urls.append("me")
    res_ls = []
    for i in urls:
        try:
            addr = ""
            if i != "me":
                if "://" in i:
                    protocol, domain = (
                        i[: i.index("://")],
                        urllib.parse.urlparse(i).netloc,
                    )
                    addr = socket.getaddrinfo(domain, protocol)[0][-1][0]
                else:  # 无网络协议或直接使用IP地址时默认采用http
                    addr = urllib.parse.urlparse(i).netloc
                    addr = socket.getaddrinfo(addr if addr else i, "http")[0][-1][0]
            res_ls.append(ip_info(addr))
        except:
            qs_default_console.print(qs_error_string, "Get domain failed:", i)
            continue
    print_ip_info(res_ls)


def wifi():
    """
    扫描附近wifi，选择连接

    Scan nearby wifi and select connection

    :return:
    """
    from . import (
        platform,
        user_lang,
        qs_default_console,
        qs_error_string,
        qs_info_string,
    )
    from .TuiTools.Table import qs_default_table

    if platform.lower() != "darwin":
        from .NetTools.WiFi import WiFi
    else:
        from .NetTools.WiFiDarwin import WiFi
    _wifi = WiFi()
    table = qs_default_table(
        ["id", "ssid", "signal", "lock"]
        if user_lang != "zh"
        else ["序号", "名称", "信号", "加密"],
        title="Wi-Fi",
    )
    connectable_wifi = _wifi.scan()
    if not connectable_wifi:
        qs_default_console.print(
            qs_error_string, "No available wifi" if user_lang != "zh" else "没有可用的wifi"
        )
        return

    from .__config__ import prompt
    from prompt_toolkit.validation import Validator, ValidationError

    class ssidValidator(Validator):
        def validate(self, document):
            document = document.text
            try:
                if int(document) >= len(connectable_wifi):
                    raise IndexError
            except:
                raise ValidationError(
                    message="请输入合法序号" if user_lang == "zh" else "Input validate id",
                    cursor_position=len(document),
                )
            return True

    questions = [
        {
            "type": "input",
            "name": "ssid",
            "message": "选择序号以连接:" if user_lang == "zh" else "Choose id to connect:",
            "default": "0",
            "validate": ssidValidator,
        },
        {
            "type": "password",
            "name": "password",
            "message": "输入密码:" if user_lang == "zh" else "Input password:",
        },
    ]

    for i, l in enumerate(connectable_wifi):
        table.add_row(*([str(i)] + [str(j) for j in l]))
    qs_default_console.print(table, justify="center")

    res = prompt(questions)
    if not res:
        return
    ssid = connectable_wifi[int(res["ssid"])]
    password = res["password"]
    qs_default_console.print(
        qs_info_string, "Connect succeed" if user_lang != "zh" else "连接成功"
    ) if _wifi.conn(ssid, password) else qs_default_console.print(
        qs_error_string, "Connect failed" if user_lang != "zh" else "连接失败"
    )


def scan_url():
    """
    探测URL对应的文件大小
    """
    from . import requirePackage

    url = sys.argv[2] if len(sys.argv) > 2 else requirePackage("pyperclip").paste()

    from .NetTools import get_fileinfo, size_format

    url, name, head = get_fileinfo(url)

    from . import qs_default_console, qs_info_string, user_lang

    qs_default_console.print(
        qs_info_string, "Name:" if user_lang != "zh" else "文件名称:", name
    )
    sz = int(head.headers.get("content-length", None))
    qs_default_console.print(
        qs_info_string,
        "Size:" if user_lang != "zh" else "文件大小:",
        size_format(sz) if sz else "Unknown" if user_lang != "zh" else "未知",
    )

    qs_default_console.print(
        qs_info_string,
        "Type:" if user_lang != "zh" else "文件类型:",
        head.headers.get("content-type", "Unknown" if user_lang != "zh" else "未知"),
    )
    qs_default_console.print(
        qs_info_string,
        "URL:" if user_lang != "zh" else "文件链接:",
        url,
    )



"""
Generate Various Table
"""


def qs_default_table(headers: list, title: str = "", without_headers: bool = False):
    """
    生成一个具有默认样式的表格

    header – 文本或可渲染的对象，用于表头。默认为 ""。

    footer – 文本或可渲染的对象，用于表尾。默认为 ""。

    header_style – 用于表头的样式，默认为 None。

    footer_style – 用于表尾的样式，默认为 None。

    style – 列单元格的样式，默认为None。

    justify – 单元格的对齐方式，默认为 "左"。

    vertical – 垂直排列, "top", "middle", 或 "bottom". 默认 "top".

    overflow – 溢出法: "crop", "fold", "ellipsis". 默认 "ellipsis".

    width – 栏目的预期宽度，以字符为单位，或无，以适应内容，默认 None.

    min_width – 列的最小宽度, 或 ``None`` 为无最低，默认 None.

    max_width – 列的最大宽度, 或 ``None`` 为无最大，默认 None.

    ratio – 列的灵活比率 (依赖 ``Table.expand`` 或 ``Table.width``)，默认 None.

    no_wrap – 设置为 "True"，可以禁用该列的包装。

    :param headers: list of str, dict or list
    :param title: title of the table
    :return:
    """
    from rich.table import Table
    from rich.box import SIMPLE_HEAVY

    res_table = Table(
        show_header=not without_headers,
        show_edge=False,
        row_styles=["none", "dim"],
        box=SIMPLE_HEAVY,
        title=title,
    )
    for i in headers:
        if isinstance(i, str):
            res_table.add_column(i, justify="center")
        elif isinstance(i, dict):
            res_table.add_column(**i)
        else:
            res_table.add_column(*i)

    return res_table



from ... import requirePackage, user_lang, qs_config
from .common import GPTBotBase

OPENAI_SETTING = qs_config.basicSelect("gpt")["support"].get("openai", None)
if not OPENAI_SETTING:
    OPENAI_SETTING = {"API_KEY": "", "API_URL": ""}
    qs_config.config["basic"]["gpt"]["support"]["openai"] = OPENAI_SETTING
    qs_config.update()

API_KEY = OPENAI_SETTING.get("API_KEY", None)
API_URL = OPENAI_SETTING.get("API_URL", None)

if not API_KEY:
    raise RuntimeError("No API key for OpenAI Chatbot.")

requirePackage("openai")
from openai import OpenAI

lang_table = {
    "zh": "Chinese",
    "en": "English",
    "fra": "French",
    "ru": "Russian",
    "spa": "Spanish",
    "ara": "Arabic",
}


class OpenAIChatBot(GPTBotBase):
    def __init__(self, model, system_prompt, record_history=True):
        super().__init__(system_prompt, record_history)
        self.model = model
        self.client = OpenAI(
            api_key=API_KEY, base_url=API_URL
        )

    def ask_stream(self, prompt):
        self.messages.append(
            {
                "role": "user",
                "content": prompt,
            }
        )
        res = self.client.chat.completions.create(
            model=self.model, messages=self.messages, stream=True
        )
        _total = ""
        for i in res:
            _cur = (i.choices[0].delta.content or '') if i.choices and len(i.choices) > 0 else ''
            _total += _cur
            yield _total
        if self.record_history:
            self.messages.append(
                {
                    "role": "assistant",
                    "content": _total,
                }
            )


def create_bot(
    model: str,
    system_prompt: str = f"You are ChatGPT, a large language model trained by OpenAI. Respond conversationally with Markdown format and using {lang_table[user_lang]} Language.",
    record_history=True
):
    return OpenAIChatBot(model, system_prompt, record_history)



# coding=utf-8
"""
命令交互，查看qs的各种菜单和文档

Command interaction, view various menus and documents of QS
"""
from . import user_lang, open_url, _ask


def onlineDocuments():
    import webbrowser as wb

    wb.open(
        "https://rhythmlian.cn/"
        + (
            "2020/02/14/QuickStart-Rhy/"
            if user_lang != "zh"
            else "2020/08/09/QuickStart-Rhy-zh/"
        )
    )


def lesson():
    try:
        res = _ask(
            {
                "type": "list",
                "message": "Welcome using qs, choose and get help:"
                if user_lang != "zh"
                else "欢迎使用qs, 选择以下选项获得帮助:",
                "choices": [
                    "1. basic tools" if user_lang != "zh" else "1. 基础工具",
                    "2. system tools" if user_lang != "zh" else "2. 系统工具",
                    "3. network tools" if user_lang != "zh" else "3. 网络工具",
                    "4. api tools" if user_lang != "zh" else "4. 扩展工具",
                    "5. image tools" if user_lang != "zh" else "5. 图像工具",
                    "6. online Docs" if user_lang != "zh" else "6. 在线文档",
                    "7. join TG group" if user_lang != "zh" else "7. 加入TG群",
                ],
            }
        )
    except KeyboardInterrupt:
        from . import qs_default_console, qs_error_string

        return qs_default_console.print(
            qs_error_string, "User Cancel" if user_lang != "zh" else "用户取消"
        )

    if res[0] == "7":
        return open_url(["https://t.me/joinchat/G2mpk7-S85eM7sb7"])
    else:
        from .funcList import menu_table

        {
            "1": menu_table["basic"],
            "2": menu_table["system"],
            "3": menu_table["net"],
            "4": menu_table["api"],
            "5": menu_table["image"],
            "6": onlineDocuments,
        }[res[0]]()



from . import pre_check, user_lang
import requests
import random

url = pre_check("DeepLX", ext=False)

if isinstance(url, list):
    _load_balancer_ = random.randint(0, len(url) - 1)

def get_url():
    """获取当前使用的API地址"""
    if isinstance(url, list):
        global _load_balancer_
        _load_balancer_ = (_load_balancer_ + 1) % len(url)
        return url[_load_balancer_]
    else:
        return url


def translate(text, target_lang=user_lang.lower()):
    """翻译文本"""
    from .. import qs_default_console

    if url:
        _url = get_url()
        res = requests.post(f"{_url}/translate", json={
            'text': text,
            'source_lang': 'auto',
            'target_lang': target_lang,
        })
        if res.text.strip():
            # return res.json().get('data')
            # get json data
            try:
                data = res.json().get('data')
                if not data:
                    from .. import qs_default_console, qs_error_string
                    qs_default_console.print(qs_error_string, f"DeepLX: {_url}")
                    return None
                return data.strip()
            except Exception as e:
                from .. import qs_default_console, qs_error_string
                qs_default_console.print(qs_error_string, f"DeepLX: {_url}")
                return None
        else:
            from .. import qs_default_console, qs_error_string
            qs_default_console.print(qs_error_string, f"DeepLX: {_url}")
            return None
    else:
        from .. import requirePackage

        translator = requirePackage("PyDeepLX", "PyDeepLX")
        if translator:
            return translator.translate(text, targetLang=target_lang)
        else:
            from .. import qs_default_console, qs_error_string
            qs_default_console.print(qs_error_string, "Get PyDeepLX failed!")
            return None

def available():
    """检查API是否可用"""
    for _ in range(len(url)):
        print(translate("Passed!"))

if __name__ == "__main__":
    # print(translate("Hello world!"))
    available()



"""
针对Mac的磁盘挂载和卸载工具
"""
from .. import external_exec


class DMG:
    """
    针对Mac的磁盘挂载和卸载工具
    """

    def __init__(self):
        self.disk_info = []

    def mount(self, path):
        """
        挂载磁盘
        """
        external_exec(f"hdiutil attach {path}")

    def unmount(self, path):
        """
        卸载磁盘
        """
        external_exec(f"hdiutil detach {path}")

    def get_disk_list(self):
        """
        获取磁盘列表
        """
        _, disk_list = external_exec("diskutil list", True)
        disk_list = disk_list.split("\n\n")
        for item in disk_list:
            item_info = item.split("\n")
            _ls = item_info[0].strip().split()
            if not _ls:
                continue
            path, disk_type = _ls[0], " ".join(_ls[1:])
            disk_type = disk_type.strip("(").strip("):")
            disk_size = 0
            for line in item_info[1:]:
                if line.strip().startswith("0"):
                    disk_size = " ".join(line.strip().split()[-3:-1])
                    break
            self.disk_info.append({"path": path, "type": disk_type, "size": disk_size})
        return self.disk_info


if __name__ == "__main__":
    from .. import qs_default_console

    disk = DMG()
    qs_default_console.print(disk.get_disk_list())



# coding=utf-8
"""
qs的API模块, 使用前确保配置文件~/.qsrc中已经配置相关的KEY值

Before using the API module of QS, make sure that the relevant key value has been configured in the configuration file ~/.qsrc
"""
from .. import qs_config, user_lang, user_root, qs_default_console, qs_error_string


def pre_check(*keys, ext: bool = True) -> str:
    """
    获取用户保存的API KEY

    Gets the API KEY saved by the user.

    :param funcName: API KEY在~/.qsrc的名称 | API KEY in the name of the .qsrc.
    :param ext: 获取失败是否退出程序 | Get failed whether to exit the program

    :return: 找到的API KEY | API KEY found.
    """
    res = []
    for key in keys:
        try:
            _val = qs_config.apiSelect(key)
            if _val is None or (isinstance(_val, str) and _val.startswith("GET:")):
                raise KeyError
        except KeyError:
            if ext:
                qs_default_console.print(
                    qs_error_string,
                    'You should set "%s" api key at: %s/.qsrc' % (key, user_root)
                    if user_lang != "zh"
                    else '你需要在qs的配置表 %s/.qsrc 中填入 "%s" 键值' % (user_root, key),
                )
                if _val:
                    from .. import qs_info_string

                    qs_default_console.print(
                        qs_info_string,
                        "How to get it:" if user_lang != "zh" else "如何获取:",
                        _val.split()[-1],
                    )
                exit(-1)
            else:
                _val = None
        res.append(_val)
    return res if len(res) > 1 else res[0]
